import argparse
from os import sep, listdir, remove
import os.path
from pathlib import Path

from metafinder.utils.banner import show_banner
from metafinder.core import processing


def main(argv=None):
    """The entry point for the script

   Args:
       argv (list): The list of parameters passed.
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('-d','--domain', help="Domain to search",required=True)
    parser.add_argument('-o','--output', help="Folder where the results will be stored",required=True, default="results")
    parser.add_argument('-l','--limit', help="Limit of documents to search", type=int, required=True)
    parser.add_argument('-t','--threads', help="Number of threads for downloading documents", type=int, default=4)
    parser.add_argument('-v','--verbose', help="Show results in terminal", action='store_true')
    args = parser.parse_args()
    show_banner()
    directory = Path(args.output) / args.domain
    directory.mkdir(parents=True, exist_ok=True)

    try:
        processing(args.domain, args.limit, args.verbose, str(directory), args.threads)
    except KeyboardInterrupt:
        print("[-] MetaFinder has been interrupted. Deleting files.")
        try:
            for f in listdir(directory):
                remove(os.path.join(directory, f))
        except:
            pass
        try:
            for f in listdir("."):
                if f.endswith(".tmp"):
                    remove(os.path.join(directory, f))
        except:
            pass
        print("Bye.")


if __name__ == '__main__':
    main(sys.argv[1:])




from os import sep


def meta_parser(metadata):
    for key, value in metadata.items():
        print("\n")
        print(key)
        print("-"* len(key))
        print(f"URL: {value['url']}")
        for v in value["metadata"]:
            if value["metadata"][v]:
                print(f'|_ {v}: {value["metadata"][v]}')


def file_parser(directory, name, metadata):
    file_name = directory + sep + name
    with open(file_name, "w") as output:
        for key, value in metadata.items():
            output.write("\n")
            output.write(key + "\n")
            output.write("-"* len(key) + "\n")
            output.write(f"URL: {value['url']}\n")
            for v in value["metadata"]:
                if value["metadata"][v]:
                    output.write(f'|_ {v}: {value["metadata"][v]}\n')

            


from os import sep, listdir, remove
import os.path
import wget
from concurrent.futures import ThreadPoolExecutor, as_completed
from metafinder.utils.file.metadata import extract_metadata

# Disable warning by SSL certificate
import ssl
ssl._create_default_https_context = ssl._create_unverified_context


def download_document(url, directory, display):
	metadata = {}
	try:
		if display:
			print(f"[+] Downloading {url}")
		document = wget.download(url, directory, bar=None)
		data = extract_metadata(document)
		if data:
			name = document.split(sep)[-1]
			metadata = {
				"name": name, 
				"url": url, 
				"metadata": data}
	except Exception as ex:
		if display:
			print(f"[-] Error ({url}): {ex}")
	return metadata


def download_file(urls_metadata, directory, threads, display=True):
	metadata_files = {}

	with ThreadPoolExecutor(max_workers=threads) as executor:
		future_download = {executor.submit(download_document, url, directory, display): url for url in urls_metadata}
		for future in as_completed(future_download):
			try:
				data = future.result()
				if data:
					name = data["name"]
					metadata_files[name] = {}
					metadata_files[name]["url"] = data["url"]
					metadata_files[name]["metadata"] =  data["metadata"]
			except Exception as exc:
				print(f"Error: {exc}")
	try:
		for f in listdir(directory):
			remove(os.path.join(directory, f))
	except:
		pass
	return metadata_files




import requests
from bs4 import BeautifulSoup
from requests.packages.urllib3.exceptions import InsecureRequestWarning
from metafinder.utils.exception import GoogleCaptcha
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)


def search(target, total):
	documents = []
	user_agent = {'User-agent': 'Mozilla/5.0 (Linux; Android 10; SM-A205U) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.86 Mobile Safari/537.36'}
	## Check https://github.com/n4xh4ck5/RastLeak - thanks Nacho
	url = f"https://www.google.com/search?q=(ext:pdf OR ext:doc OR ext:docx OR ext:xls OR ext:xlsx OR ext:ppt OR ext:pptx)+(site:*.{target} OR site:{target})&filter=0&num={total}"
	try:
		response = requests.get(url, headers=user_agent)
		text = response.text
		soup = BeautifulSoup(text, "html.parser")
		if text.find("detected unusual traffic") != -1:
			raise GoogleCaptcha()
		all_links = soup.find_all("a")

		for link in all_links:
			href = link.get("href", None)
			if href and target in href and \
			(href.endswith("pdf") or href.endswith("doc") or href.endswith("docx") or href.endswith("ppt") or href.endswith("pptx") or href.endswith("xls") or href.endswith("xlsx")) \
			and href not in documents:
				documents.append(href)
	except Exception as ex:
		#It's left over... but it stays there
		raise ex
	return documents	



from metafinder.utils.finder import google
from metafinder.utils.file.download import download_file
from metafinder.utils.file.parser import meta_parser, file_parser


def processing(target, limit, verbose, directory, threads):
    try:
        links = google.search(target, limit)
    except Exception as ex:
        print(ex)
        return
    total_links = len(links)
    print(f"Total files to be analyzed: {total_links}")
    print("-------------------------------")
    if total_links > 0:
        metadata_files = download_file(links, directory, threads)
        print("[+] Analyzing metadata...")
        if metadata_files:
            if verbose:
                meta_parser(metadata_files)
            file_parser(directory, "metadata_google.txt", metadata_files)
            print(f"[+] The results have been saved in the file {directory}/metadata_google.txt")
        else:
            print("[-] No metadata found...")
    else:
        print("[-] There is nothing to analyze. Closing...")




class GoogleCaptcha(Exception):
    def __init__(self, *args):
        self.data = "Google Captcha detected"
        if args:
            self.data = args[0]
    def  __str__(self):
        return "GoogleCaptcha, {0}".format(self.data)



from setuptools import setup, find_packages
from os import path
import metafinder


setup(
    name='metafinder',
    version=metafinder.__version__,
    author='Josue Encinar (@JosueEncinar)',
    description='MetaFinder - Metadata search through Google',
    include_package_data=True,
    license='GNU GPLv3+',
    packages=find_packages(),
    url="https://github.com/Josue87/MetaFinder",
    entry_points={
        'console_scripts': [
            'metafinder=metafinder.cli:main',
        ],
    },
    install_requires=[
        "requests",
        "pypdf2",
        "beautifulsoup4",
        "openpyxl",
        "wget",
        "python-docx",
        "python-pptx"
    ]
)



from pathlib import Path
import tempfile
from os.path import isfile
from metafinder.utils.finder import google
from metafinder.utils.file.download import download_file
from metafinder.utils.file.metadata import extract_metadata


def extract_metadata_from_google_search(domain, limit=100, threads=4):
    """Search metadata in files through Google

    Args:
       domain: Target domain.
       limit: Maximum number of files to search.
       threads: Threads for downloading documents.
    
    Raises:
        GoogleCaptcha: If Google displays the captcha.
        Exception: If there is an error

    Returns:
        dict: A json with metadata
    """
    links = google.search(domain, limit)
    directory = tempfile.TemporaryDirectory()
    metadata_files = None
    if len(links) > 0:
        metadata_files = download_file(links, directory.name, threads, False)
    directory.cleanup()
    return metadata_files

def extract_metadata_from_document(document):
    """Search metadata in a document

    Args:
       document: Document to be analyze.

    Raises:
        FileNotFoundError: If file doesn't exist.

    Returns:
        dict: A json with metadata
    """
    if isfile(document):
        return extract_metadata(document)
    raise FileNotFoundError()



from docx import Document
from PyPDF2 import PdfFileReader
from openpyxl import load_workbook
from pptx import Presentation


def extract_doc(document):
    metadata = {}
    doc = Document(document)
    prop = doc.core_properties
    metadata["Author"] = prop.author
    metadata["Comments"] = prop.comments
    metadata["Created"] = prop.created
    metadata["Identifier"] = prop.identifier
    metadata["Keywords"] = prop.keywords
    metadata["Modified"] = prop.modified
    metadata["Subject"] = prop.subject
    metadata["Title"] = prop.title
    return metadata

def extract_pdf(document):
    metadata = {}
    with open(document, 'rb') as f:
        pdf = PdfFileReader(f, strict=False)
        info = pdf.getDocumentInfo()
        for meta in info:
            data = info.get(meta, None)
            metadata[meta[1:]] = data
    return metadata


def extract_xls(document):
    metadata = {}
    wb = load_workbook(document)
    prop = wb.properties
    metadata["Author"] = prop.creator
    metadata["Comments"] = prop.description
    metadata["Created"] = prop.created
    metadata["Identifier"] = prop.identifier
    metadata["Keywords"] = prop.keywords
    metadata["Modified"] = prop.modified
    metadata["Subject"] = prop.subject
    metadata["Title"] = prop.title
    return metadata

def extract_ppt(document):
    pptx_presentation = Presentation(document)
    prop = pptx_presentation.core_properties
    metadata["Author"] = prop.author
    metadata["Comments"] = prop.comments
    metadata["Created"] = prop.created
    metadata["Identifier"] = prop.identifier
    metadata["Keywords"] = prop.keywords
    metadata["Modified"] = prop.modified
    metadata["Subject"] = prop.subject
    metadata["Title"] = prop.title
    return metadata


def remove_indirect_object(metadata):
    new_metadata = {}
    for m in metadata:
        value =  metadata[m]
        if "IndirectObject(" not in str(value) and str(value) != "":
            new_metadata[m] = value

    return new_metadata


def extract_metadata(document):
    try:
        if document.endswith("pdf"):
            metadata = extract_pdf(document)
        elif document.endswith("doc") or document.endswith("docx"):
            metadata = extract_doc(document)
        elif document.endswith("xls") or document.endswith("xlsx"):
            metadata = extract_xls(document)
        elif document.endswith("ppt") or document.endswith("pptx"):
            metadata = extract_ppt(document)
        else: 
            metadata = {}
    except:
        metadata = {}
  
    return remove_indirect_object(metadata)




